function calc_ellipse(filenm,par_out,parname1,parname2,varargin)

% Usage: calc_ellipse(filenm,par_out,parname1,parname2,varargin)
%
% Calculate and plot an error ellipse that covers 95% of the parameter
% values from the MCMC sample, saved by calc_slice.m. Alternatively, input
% from the LHS sample of the joint likelihood-based region, as saved by
% calc_likregion.m.
%
% Note that the ellipse is calculated from the mean and covariance in the
% sample, so it represents a normal approximation. This is done for two
% selected parameters, indicated by parname. Uses the function
% error_ellipse. This is NOT done for the likelihood-based region, as the
% sample does not represent the posterior distribution.
%
% Author : Tjalling Jager
% Date: Aug 2015
% Web support: http://www.debtox.info/byom.html

global names
if isempty(names) % if this script is run without fitting, 'names' is  not defined
    names  = fieldnames(par_out); % extract all field names of par (global)
end
if ~isfield(par_out,parname1)
    error('The first parameter name you entered is not correct.')
end
if ~isfield(par_out,parname2)
    error('The second parameter name you entered is not correct.')
end
if isempty(varargin)
    type_conf = 1; % assume Bayesian MCMC by default
    % this ensures compatibility for older BYOM script files
else
    type_conf = varargin{1}; % type of analysis is provided
end

switch type_conf
    case 1 % Bayesian sample from the posterior
        load([filenm,'_MC']) % load the random sample from the last MCMC run
    case 2 % LHS sample from the joint CI (accepted values only)
        load([filenm,'_LR']) % load the random sample from the last MCMC run
    case 3 % Bayesian sample from the posterior (accepted values only)
        load([filenm,'_MC']) % load the random sample from the last MCMC run
end

%% Extracting the right part of the sample, for the two selected parameters

pmat   = packunpack(1,par_out,0); % transform structure into a regular matrix
nfit   = pmat(:,2)==1; % ones for fitted parameters
% some parameters in pmat may be on log-scale, if they are fitted that way

bestfit = eval(['[par_out.',parname1,'(1) par_out.',parname2,'(1)]']);
logfit = eval(['[par_out.',parname1,'(5) par_out.',parname2,'(5)]']); % are they fitted on log scale?
if any(logfit==0) % put the best parameter estimate on log-scale
    bestfit(logfit==0) = log10(bestfit(logfit==0));
end

ind1 = 0;
ind2 = 0;
for i = 1:length(nfit)
    if strcmp(names(i),parname1) && nfit(i) == 1
        ind1 = sum(nfit(1:i));
    end
    if strcmp(names(i),parname2) && nfit(i) == 1
        ind2 = sum(nfit(1:i));
    end
end
if ind1 == 0 || ind2 ==0
    error('The parameters you entered for the ellipse are not available in the random sample.')
end
    
rnd2 = rnd(:,[ind1 ind2]); % these are the columns we asked for (all values)
acc2 = acc(:,[ind1 ind2]); % these are the columns we asked for (accepted values only)

mu     = mean(rnd2); % center of the ellipse
covmat = cov(rnd2);  % covariance matrix for the sample

%% Plot the sample in a 2-d plot (for the two selected parameters)

figure
hold on

switch type_conf
    case 1 % for Bayesian sample only
        error_ellipse(covmat,mu,'conf',0.95)
        title('Random sample from the posterior distribution')
        leg = {'asymptotic region'};
        % the error ellipse is an asymptotic approximation of the joint CR
    case 2 % for likelihood-based region
        title('Sample from parameter space, showing joint 95% lik.-based conf. region')
        leg = {};
    case 3 % for Bayes, focus on accepted sample
        error_ellipse(covmat,mu,'conf',0.95)
        title('Random sample from the posterior distribution')
        leg = {'asymptotic region'};
        % the error ellipse is an asymptotic approximation of the joint CR
end

h_all = plot(rnd2(:,1),rnd2(:,2),'ko','MarkerSize',1,'MarkerFaceColor','k'); % plot total sample
plot(acc2(:,1),acc2(:,2),'ko','MarkerFaceColor','w'); % plot accepted values (within 95% conf. region)
plot(bestfit(1),bestfit(2),'ko','MarkerFaceColor','y','MarkerSize',10); % plot the best fit par. set

leg = [leg;{'outside 95% region';'within 95% region';'best fit set'}]; % add text to the legend

% convex hull that captures the 95% edges of the highest probability confidence region
k = convhull(acc2(:,1),acc2(:,2));
plot(acc2(k,1),acc2(k,2),'r-')
leg = [leg;{'outer edge 95% region'}]; % add to the legend
legend(leg) % create legend
uistack(h_all,'bottom') % make sure the total sample is at the bottom

% add text to axes
if logfit(1) == 0,
    xlabel(['parameter ',names{ind1},' (log10-scale)'],'FontSize',12,'FontName','Arial')
else
    xlabel(['parameter ',names{ind1}],'FontSize',12,'FontName','Arial')
end
if logfit(2) == 0,
    ylabel(['parameter ',names{ind2},' (log10-scale)'],'FontSize',12,'FontName','Arial')
else
    ylabel(['parameter ',names{ind2}],'FontSize',12,'FontName','Arial')
end
set(gca,'LineWidth',1,'FontSize',12,'FontName','Arial')

