function [rnd,coll_rndsort] = calc_slice(par,nrs,filenm)

% Usage: calc_slice(par,nrs,filenm)
%
% Calculation of posterior probability (optional informative priors in
% script file) using MCMC (the slicesample algorithm in the statistics
% toolbox. Marginal densities are calculated from the sample using the
% kernel density estimation in the statistics toolbox (ksdensity).
% Confidence intervals are calculated from the densities by finding the
% parameter values with the highest densities that together yield 95% of
% the total area under the curve. This means that 'islands' of likelihood
% are also captured in the confidence intervals. Additionally, quantiles
% are provided.
%
% nrs is a vector with the number of samples drawn, the thinning of the
% sample, and the number of burn-in samples. This function saves the random
% sample into a file with as name: filenm_MC.MAT (filenm is the name of the
% mydata file that called the slice sampler). This saved file can be used
% to calculate prediction intervals and error ellipses.
%
% Author : Tjalling Jager
% Date: Aug 2015
% Web support: http://www.debtox.info/byom.html

global names pri namesp

if exist('slicesample','file')~=2, % only when slicesample exists as an m-file in the path
    error('No posterior distribution calculation; you need the statistics toolbox for that.')
end

disp(' ')
disp('Calculating sample from posterior density ... please be patient.')
drawnow % plot the last plot in the plot buffer before starting the analysis

% trim parametervectors to use only the reduced set of parameters
pmat = packunpack(1,par,0);  % transform globals structure into a regular matrix

% put parameters that need to be on log scale on log10 scale
pmat(pmat(:,5)==0,1) = log10(pmat(pmat(:,5)==0,1));
% do NOT modify the min max ranges, as transfer needs them on normal scale

p_index = find(pmat(:,2)==1);
parshat = pmat(p_index,1); % Only the parameters that were fitted
par_sel = p_index;

%% Run slicesample.m (statistics toolbox) for the MCMC sample

rnd = slicesample(parshat,nrs(1),'logpdf',@(pars) -1 * transfer(pars,pmat),'thin',nrs(2),'burnin',nrs(3));
% transfer gives the min log likelihood, here we need the log likelihood, so multiply by -1

figure
hold on
for i = 1:length(parshat),
    subplot(length(parshat),1,i),plot(rnd(:,i)) % plot the random sample in raw form
    [ind1,ind2] = ind2sub(size(par_sel),p_index(i)); % index to the fitted parameter in 2 dims
    subplot(length(parshat),1,i),ylabel(names{p_index(i)},'FontSize',10,'FontName','Arial')
end

%% Plot marginal distributions and calculate credible intervals

corr_rnd = corrcoef(rnd); % correlation matrix from the sample
rndsort  = sort(rnd); % sort the MCMC sample
% cdf_x = linspace(0,1,size(rnd,1)); % for cumulative density plots, if needed

diary ('results.out') % collect output in the diary "results.out"
disp(' ')
disp('Credible intervals (95% highest posterior density, and 2.5-97.5% quantiles)')
disp('======================================================================================')

for parnr = 1:length(p_index), % run through all fitted parameters
    [ind1,ind2] = ind2sub(size(par_sel),p_index(parnr)); % index to the fitted parameter in 2 dims
    
    figure % ('Position',[100   100   560   420])
    hold on
    % title([TYPES.name,': ',ADD(ind2).studyname,' (',date,')'],'FontSize',10,'FontName','Arial')
    
    if pmat(p_index(parnr),5) == 0 % if parameter was fitted on log-scale, the plot is also of the log values
        xlabel(['parameter ',names{p_index(parnr)},' (log-scale)'],'FontSize',12,'FontName','Arial')
        supp_par = log10(pmat(p_index(parnr),[3 4])); % support for kernel density is min-max on log scale
    else
        xlabel(['parameter ',names{p_index(parnr)}],'FontSize',12,'FontName','Arial')
        supp_par = pmat(p_index(parnr),[3 4]); % support for kernel density is min-max on normal scale
    end
    ylabel('estimated density','FontSize',12,'FontName','Arial')
    set(gca,'LineWidth',1,'FontSize',12,'FontName','Arial')
    % plot(rndsort(:,parnr),cdf_x,'k-') % plot cumulative density, if needed
    
    [f,x,u] = ksdensity(rndsort(:,parnr),'support',supp_par); % use kernel density estimation
    % Note: limit support for the posterior to the min-max range provided in the byom script
    plot(x,f,'k-'); % plot the estimated marginal probability density for the parameter
    
    if isfield(pri,names{p_index(parnr)}) % is there a prior for this one?
        namesp_rem = namesp; % make a copy of the prior names
        namesp{1}  = names{p_index(parnr)}; % make this one the only name in the global
        
        % find a reasonable range to plot the prior on
        a = eval(['pri.',names{p_index(parnr)}]); % extract prior info
        switch a(1)
            case 1
                pribnds = [0 1];
            case 2
                pribnds = [a(2) a(3)];
            case 3
                pribnds = [a(2)-2*a(3) a(2)+2*a(3)];
            case 4
                pribnds = exp([a(2)-2*a(3) a(2)+2*a(3)]);
        end
        if pmat(p_index(parnr),5) == 0 % if parameter is fitted on log-scale
            pribnds(1) = max(0,1e-10);
            pribnds = log10(pribnds); % put them on log scale
        end
        
        % set plotting range to a reasonable range to catch the entire
        % posterior and a relevant part of the prior
        rangep = linspace(min(x(1),max(pribnds(1),x(1)-abs(x(1)/2))),max(x(end),min(pribnds(2),x(end)+abs(x(end)*2))),100); % range of values for plotting prior
        priprob = zeros(length(rangep),1);
        for ip = 1:length(rangep)
            if pmat(p_index(parnr),5) == 0 % if parameter is fitted on log-scale
                eval(['ptmp.',namesp{1},'=10.^rangep(ip);']);
            else
                eval(['ptmp.',namesp{1},'=rangep(ip);']);
            end
            priprob(ip) = exp(calc_prior(ptmp)); % call function to calculate prior prob.
        end
        
        plot(rangep,priprob,'r-')
        namesp = namesp_rem; % and return the full namesp vector as a global
        xlim([rangep(1) rangep(end)])
        
    end
    
    % Next, find the critical density for the 95% highest-prob. region, using fzero
    stepsize = x(2)-x(1);
    clear coll_int
    critdens = fzero(@(testdens) sum(f(f>=testdens))*stepsize-0.95,max(f)/2); % this is the prob density that leads to 95% of the values
    plot([min(x) max(x)],[critdens critdens],'k:') % plot this cut off prob density as a horizontal line
    axis tight % axis as close as possible
    
    % Now calculate the 95% highest-prob. boundaries
    clear coll_int
    testrange = f-critdens;
    direction = -1; % f-critdens starts as negative (we always first find the start of an interval)
    nr_int = 0;
    for i = 1:length(x),
        if direction == -1 && testrange(i) > 0, % we found the start of an interval
            nr_int = nr_int+1;
            if i > 1,
                coll_int(nr_int,1) = mean([x(i) x(i-1)]); % take average as start of interval
            else
                coll_int(nr_int,1) = x(1); % take first as start of interval
            end
            direction = 1; % change the direction to spot end of interval
        end
        if direction == 1 && testrange(i) < 0, % we found the end of an interval
            if i<length(x),
                coll_int(nr_int,2) = mean([x(i) x(i-1)]); % take average as end of interval
            else
                coll_int(nr_int,2) = x(end); % take last as end of interval
            end
            direction = -1; % change the direction to spot start of new interval
        end
    end
    
    % Calculate the quantiles

    %     coll_rndsort(parnr,:) = quantile(rndsort(:,parnr),[0.025 0.975]); % numbers for quantiles
    %     % plot broken lines to indicate quantiles
    %     plot([coll_rndsort(parnr,1) coll_rndsort(parnr,1)],[0 interp1(x,f,coll_rndsort(parnr,1))],'k--')
    %     plot([coll_rndsort(parnr,2) coll_rndsort(parnr,2)],[0 interp1(x,f,coll_rndsort(parnr,2))],'k--')
    
    % using the kernel densities for the quantiles is better for small samples
    if any(f<1e-20) % sometimes there are zeros or very low numbers in there!
        f = f+1e-10; % add a small amount should prevent error...
    end
    coll_rndsort(parnr,1) = interp1(cumtrapz(x,f),x,0.025);
    coll_rndsort(parnr,2) = interp1(cumtrapz(x,f),x,0.975);
    % If this still yields an error: try with more samples or use the commented-out code above
    
    % plot broken lines to indicate quantiles
    plot([coll_rndsort(parnr,1) coll_rndsort(parnr,1)],[0 interp1(x,f,coll_rndsort(parnr,1))],'k--')
    plot([coll_rndsort(parnr,2) coll_rndsort(parnr,2)],[0 interp1(x,f,coll_rndsort(parnr,2))],'k--')
        
    if pmat(p_index(parnr),5) == 0 % parameter is on log scale
        coll_rndsort = 10.^coll_rndsort; % for screen, make output normal scale
        coll_int = 10.^coll_int;
    end
    
    if nr_int == 1, % no problems, it is a single interval
        % fprintf('%-5s (%1.0f) interval: %10.4g - %10.4g quant.: %10.4g - %10.4g\n',PARS{ind1,1},ind2,coll_int,coll_rndsort(parnr,:))
        fprintf('%-10s interval: %10.4g - %10.4g quant.: %10.4g - %10.4g\n',names{p_index(parnr)},coll_int,coll_rndsort(parnr,:))
    else
        disp('The credible interval is a broken set (check density plots to check these figures)')
        fprintf('%-10s                                   quant.: %10.4g - %10.4g\n',names{p_index(parnr)},coll_rndsort(parnr,:))
        for i_broken = 1:nr_int,
            fprintf('           interval: %10.4g - %10.4g \n',coll_int(i_broken,:))
        end
    end
end

disp('======================================================================================')
disp(' ')
disp('Correlation matrix from the MCMC sample')
disp('======================================================================================')
fprintf('%5s ',['  '])
for i = 1:length(p_index),
    [inda,indb]=ind2sub(size(par_sel),p_index(i)); % finds the subscripts from the indices of this parameter
    fprintf('%11s ',names{p_index(i)})
end
fprintf('\n')
for i = 1:length(p_index),
    [inda,indb]=ind2sub(size(par_sel),p_index(i)); % finds the subscripts from the indices of this parameter
    fprintf('%-5s ',names{p_index(i)})
    for j = 1:i, % length(p_index),
        fprintf('%11.4g ',corr_rnd(i,j))
    end
    fprintf('\n')
end
if any(pmat(:,5)==0)
    disp('Note: some parameters have been estimated on log-scale; correlations are also on log-scale for these parameters.')
end
disp('======================================================================================')

%% Find the joint 95% highest-prob. region
% Here, I calculate the log-likelihood for each parameter set to be able to
% construct a joint, highest-probability, credible region! This is also
% saved to the file. It is NOT used by calc_conf, but it IS used by
% calc_ellipse for plotting two-dimensional joint intervals.

for i = 1:size(rnd,1) % run through all samples
    loglik_rnd(i) = -1 * transfer(rnd(i,:),pmat); % use transfer to obtain max likelihood!
end % pmat includes the log parameters on log scale, as transfer needs
meanloglik = mean(loglik_rnd); % use the mean as a starting point for finding 
         % the critical log-lik corresponding to 95% of the random sample
critloglik = fzero(@(testloglik) sum(loglik_rnd>testloglik)/length(loglik_rnd)-0.95,meanloglik); 
% this is the log likelihood that captures 95% of the values from the sample
ind_acc = loglik_rnd>=critloglik; % index to accepted values (within the 95% credible region)
acc     = rnd(ind_acc,:); % these are the accepted parameters (within the 95% credible region)

disp(' ')
disp(['Number of MCMC samples taken: ',num2str(nrs(1)),', of which accepted in the credible region: ',num2str(size(acc,1))])

% save the sample in a MAT-file with the name of the script, with
% _MC at the end. Load can be used to retrieve it.
% also, save par (total parameter matrix) and par_sel (selection
% matrix) to make plots without redoing the fit.
save([filenm,'_MC'],'rnd','acc','par','par_sel')

disp(['Time required: ' secs2hms(toc)])
diary off    % close results.out