function dX = derivatives(t,X,par,c)

% This function calculates the derivatives for the model system. As input,
% it gets:
% <t>   the time point, provided by the ODE solver
% <X>   a vector with the previous value of the states
% <par> the parameter structure
% <c>   the external concentration (or scenario number)
%
% Parameters <t> and <c> are available, but you do not have to use them.
%
% Output <dX> (as vector) provides the differentials for each state at <t>.
% This function is called by call_deri.m.
%
% Author: Tjalling Jager (www.debtox.info)
% Date  : Dec 2014

global glo          % allow for global parameters in structure glo (handy for switches)
global pp_coll pp_scen % for splined external concentrations

% =======================================================================
%                  UNPACK STATES FOR EASY REFERENCE
% =======================================================================

S  = X(1); % state 1 is the survival probability at previous time point
Ci = X(2); % state 2 is the scaled internal concentration at previous time point

% =======================================================================
%                  UNPACK PARAMETER STRUCTURE FOR EASY REFERENCE
% =======================================================================

ke   = par.ke(1);   % elimination rate constant, d-1
if isfield(par,'K') % if a bioconcentration factor is specified, use it
    K = par.K(1);   % bioconcentration factor, L/kg (set to 1 for scaled)
else
    K = 1;          % otherwise, set to 1 (scaled model)
end
z    = par.z(1);    % no-effect concentration, ug/L or ug/kg
kk   = par.kk(1);   % killing rate, L/ug/d or kg/ug/d
Fs   = par.Fs(1);   % fraction spread of NEC distribution, (-) (used in call_deri)
hb   = par.hb(1);   % background hazard rate, d-1

% =======================================================================
%                  NOW CALCULATE THE DERIVATIVES
% =======================================================================

if ~isempty(pp_coll) % if it exists: use it to interpolate external conc.
    [pp_exist,pp_loc] = ismember(c,pp_scen); % is c in the global?
    if pp_exist == 1 
        if exist('griddedInterpolant') == 2 % for new versions of Matlab
            c = pp_coll{pp_loc}(t); % use the pp form already provided
        else
            c = ppval(pp_coll{pp_loc},t); % use the pp form already provided
        end
    end
end

dCi = ke * (K * c - Ci); % first order bioconcentration

switch glo.sel
    case 1 % stochastic death
        h = kk * max(0,Ci-z); % calculate the hazard rate
        dS = -(h + hb)* S; % change in survival probability (incl. background mort.)
    case 2 % individual tolerance
        dS = -hb* S; % only background hazard rate
        % mortality due to the chemical is included in call_deri!
    case 3 % mixed model
        dS = -hb* S; % only background hazard rate
        % mortality due to the chemical is included in call_deri!
end

dX = [dS;dCi]; % collect both derivatives in one vector
