function Xout = simplefun(t,X0,par,c)

% This function calculates an explicit funtion  for the model system. As input,
% it gets:
% <t>   the time vector, provided by call_deri
% <X0>  a vector with the initial value of the states (may not be needed)
% <par> the parameter structure
% <c>   the external concentration (or scenario number)
%
% Parameters <t> and <c> are available, but you do not have to use them.
%
% Output <Xout> (as matrix) provides the output for each state at <t>.
% This function is called by call_deri.m.
%
% Author: Tjalling Jager (www.debtox.info)
% Date  : Dec 2014

global glo   % allow for global parameters in structure glo (handy for switches)

% This calculation of survival only works when the exposure concentration
% is constant over time!

% =======================================================================
%                  UNPACK INITAL STATES FOR EASY REFERENCE
% The state variables enter this function in the vector "X"
% =======================================================================

S0  = X0(1); % survival probability at t=0
Ci0 = X0(2); % scaled internal concentration at t=0

% =======================================================================
%                  UNPACK PARAMETER STRUCTURE FOR EASY REFERENCE
% The parameters enter this function in the structure "par". The names in
% the structure are the same as those defined in the byom script file.
% =======================================================================

ke   = par.ke(1);   % elimination rate constant, d-1
if isfield(par,'K') % if a bioconcentration factor is specified, use it
    K = par.K(1);   % bioconcentration factor, L/kg (set to 1 for scaled)
else
    K = 1;          % otherwise, set to 1 (which means the scaled model)
end
z    = par.z(1);    % no-effect concentration, ug/L
kk   = par.kk(1);   % killing rate, L/ug/d
Fs   = par.Fs(1);   % fraction spread of NEC distribution (used in call_deri)
hb   = par.hb(1);   % background hazard rate, d-1

% =======================================================================
%                  NOW CALCULATE THE OUTPUT
% =======================================================================

S  = exp(-hb*t); % initialise S with background mortality for survival
Ci = Ci0*exp(-ke*t)+(1-exp(-ke*t)) * c * K;  % internal concentration over time

if glo.sel == 1, % only for stochastic death
    z  = z / K;  % needed as the analytical solution is for the scaled model
    kk = kk * K; % needed as the analytical solution is for the scaled model
    
    % This is the same calculation as in DEBtool fomort
    f  = zeros(length(t),1);
    t0 = -log(max(1e-8,1-z/max(1e-8,c)))/ke; % no-effect-time
    t1 = ones(length(t),1);                   % column-matrix of ones
    f  = (kk/ke)*max(0,t1*exp(-ke*t0) - exp(-ke*t)).*(t1*c) - ...
        kk*(t1*(max(0,c-z)')).*max(0,t - t1*t0);
    
    S = min(1,exp(f)) .* S; % multiply with background mortality
end

% For individual tolerance, the calculation is done in call_deri and only
% background survival is calculated here. Same for mixed model.
   
Xout = [S Ci]; % combine them into a matrix